/*  Juhtmevaba iselaadiv robot
 *   
 *  Programmi abil tuvastab robot aku tühjenemise ning
 *  läheb laadimispunkti juhtmevabalt laadima
 *  
 *  Autor: Triinu Liis Kelder
 *  Programmis on kasutatud näidisprogramme, mis on avalikult kättesaadavad https://www.arduino.cc/en/Tutorial/BuiltInExamples
 *  ning http://metshein.com/index.php/arvuti/arduino
 *  
 *  2016
*/

#include <IRremote.h>
#include <IRremoteInt.h>
// https://arduino-info.wikispaces.com/IR-RemoteControl
// RobotIRRemote tuleb ära võtta (C:\Program Files (x86)\Arduino\libraries\RobotIRremote), et saaks seda teeki kasutada

#include <Servo.h>  // Arduino IDE-s olemas

#include <Wire.h> // Arduino IDE-s olemas
#include <LiquidCrystal_I2C.h>  // Tuleb eraldi installida (https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads)

// LCD ekraan
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Ühendab LCD-ekraani õigete pesadega ära

// Servo
Servo servoLeft;  // Loob servo objekti
Servo servoRight;  // Loob servo objekti

// Aku pinge leidmine
const int analogInPin = A0;  // Määrab ära, millisest pesast pinge väärtus loetakse
int sensorValue = 0;        // Loob sensorValue muutuja

// IR vastuvõtja
int RECV_PIN = 11;  // Määrab ära sensori pesa
IRrecv irrecv(RECV_PIN); // Teeb irrecv objekti
decode_results results; // Teeb results objekti

int no_signal = 0;

// Ultraheli
int EchoPin = 12; // Määrab õiged pesad
int TrigPin = 13;
long EchoTime = 0;

bool otsi = false;  // Muutuja, kas tuleb otsida laadimispunkti või ei

bool paremale = true; // Muutuja, kas tuleb pöörata paremale või mitte
int turn_counter = 0; // Muutuja, palju on robot pööranud
bool leitud = false;  // Muutuja, kas laadimispunkti signaal on leitud või mitte
bool kohal = false;   // Muutuja, kas robot on jõudnud laadimispunkti või mitte

int takistus = false;  // Muutuja, kas takistus on lähedal või ei

void setup() {
  Serial.begin(9600); // Initsialiseerib jadapordi monitori
  servoLeft.attach(7);  // Ühendab servo objekti pesaga 7
  servoRight.attach(9);  // Ühendab servo objekti pesaga 9
  lcd.begin (16,2); // Initsialiseerib ekraani 16 veeru ja 2 reaga
  irrecv.enableIRIn(); // Alustab IR vastuvõtja sensori
  
  // Ultrasonic
  pinMode(EchoPin, INPUT);
  pinMode(TrigPin, OUTPUT);
  stopRobot();
}

void loop() {
  lcd.setCursor(0, 0);
  lcd.backlight();
  readVoltage();
  for (int i = 0; i < 40; i++) {
    is_signal();
    delay(50);
  }  
}

// Väljastab true kui robot on liiga lähedal takistusele ning false kui ei ole veel liiga lähedal
int ultrasonic() {
  digitalWrite(TrigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(TrigPin, LOW);
 
  EchoTime = pulseIn(EchoPin, HIGH);
  int dist = (EchoTime/2)/29; 
  Serial.print(dist, DEC);
  Serial.println("cm");
  return dist;
}

void is_signal() {
  if (irrecv.decode(&results)) {
    switch(results.value) {
      case 0xa90: // Signaal, mis tuleb infrapuna saatjast
        // Kui tuleb otsida, kontrollib, kas takistus on lähedal
        // Kui takistus on kaugel, liigub edasi 50 ms
        // Kui takistus lähedal, peatab roboti
        if (otsi) {
          leitud = true;
          // Kontrollib, ega robot liiga lähedal laadimispunktile pole
          if (ultrasonic() > 15 || ultrasonic() == 0) {
            forward(50);
          // Robot on peaaegu laadimispunktis
          // Edasi liigub ainult otse, kuni jõuab laadijani
          } else {
            while (ultrasonic() > 2) {
              forward(50);  // Robot läheb edasi kuni jõuab laadimispunktist 2 cm kaugusele
            }
            forward(20);  // Natukene veel, et oleks vastu laadijat
            kohal = true;  // Laadimispunkt on leitud
            otsi = false; // Otsida pole enam vaja
            stopRobot();
          }
          no_signal = 0;
        }
        break;
      case 0xFD8877:  // Üles nool puldil
        forward(200);
        stopRobot();
        break;
      case 0xFD6897:  // Paremale nool puldil
        turnRight(400);
        stopRobot();
        break;
      case 0xFD28D7:  // Vasakule nool puldil
        turnLeft(400);
        stopRobot();
        break;
      case 0xFD9867:  // Alla nool puldil
        reverse(200);
        stopRobot();
        break;
      case 0xFDA857:  // OK nupp puldil
        stopRobot();
        break;
      case 0xFD30CF:  // * nupp puldil
        otsi = true;
        break;
      case 0xFD708F:  // # nupp puldil
        otsi = false;
        break;
      default:  // Mingi teine väärtus leiti
        // Kui tuleb otsida, peatab roboti ja suurendab no_signal väärtust
        if (otsi) {
          stopRobot();
          no_signal++;
        }
        Serial.println(results.value, HEX);
    }
  } else {  // Signaali ei leitud
    // Tuleb otsida, aga signaali ei leitud.
    // Peatab roboti ja suurendab no_signal väärtust
    if (otsi) {
      stopRobot();
      no_signal++;
    }
  }
  irrecv.resume();  // Otsib uuesti signaali

  // Robot liigub
  if (!kohal) {
    liigu();
  }
}

void liigu() {
  takistus = ultrasonic();
  if (takistus > 10 || takistus == 0 || leitud) {  // Kas takistus on lähedal
    // Kui signaali pole leitud 10 korda järjest ja tuleb otsida ja pole täisringi veel teinud
    if (no_signal > 9 && (otsi || leitud) && turn_counter < 120) {
      turnRight(40);
      turn_counter++;
      stopRobot();
    } else {
      forward(50);
      turn_counter = 0;
    }
  } else {
    stopRobot();
    if (paremale) { // Tuleb pöörata paremale
      turnRight(400); // Pöörab paremale
      stopRobot();
      takistus = ultrasonic();
      if (takistus > 10 || takistus == 0) {  // Kontrollib, kas on taksitus lähedal
        forward(50);
        turnRight(400);
        stopRobot();
        paremale = false;
      } else {  // Kui takistus tuli liiga lähedale, proovib vasakule pöörata
        turnLeft(800);
        stopRobot();
        takistus = ultrasonic();
        if (takistus > 10 || takistus == 0) {  // Kontrollib, kas on taksitus lähedal
          turnLeft(350);
          stopRobot();
        } else {  // Kui takistus tuli liiga lähedale, pöörab tagasi otse, liigub tagasi
          turnRight(400);
          stopRobot();
          reverse(50);
        }
      }
    } else { // Tuleb pöörata vasakule
      turnLeft(400);
      stopRobot();
      takistus = ultrasonic();
      if (takistus > 10 || takistus == 0) {  // Kontrollib, kas on taksitus lähedal
        forward(50);
        turnLeft(400);
        stopRobot();
        paremale = true;
      } else {  // Kui takistus tuli liiga lähedale, proovib paremale pöörata
        turnRight(700);
        stopRobot();
        takistus = ultrasonic();
        if (takistus > 10 || takistus == 0) {  // Kontrollib, kas on taksitus lähedal
          turnRight(400);
          stopRobot();
        } else {  // Kui takistus tuli liiga lähedale, pöörab tagasi otse, liigub tagasi
          turnLeft(400);
          stopRobot();
          reverse(50);
        }
      }
    }
  }
}

// Võtab argumendiks rea numbri ning
// kustutab sellel real oleva sõne
void emptyAndSetRow(int row) {
  lcd.setCursor(0, row);
  lcd.print("                ");
  lcd.setCursor(0, row);
}

// Loeb aku pinget ning väljastab selle LCD-ekraanile
void readVoltage() {
  sensorValue = analogRead(analogInPin);
  // Kui aku pinge on alla 3.6 voldi, hakkab robot otsima laadimispunkti
  if ((sensorValue / 1023.0 * 5.0) < 3.6) {
    otsi = true;
  }
  // Väljastab LCD-ekraanile aku pinge voltides
  emptyAndSetRow(0);
  lcd.print(sensorValue / 1024.0 * 5.0);
  lcd.print("V");
}

// Võtab argumendiks täisarvu ning
// sõidab nii kaua edasi (millisekundites)
void forward(int n) {
  servoLeft.write(180);
  servoRight.write(7);
  delay(n);
}

// Võtab argumendiks täisarvu ning
// sõidab nii kaua tagasi (millisekundites)
void reverse(int n) {  
  servoLeft.write(7);
  servoRight.write(180);
  delay(n);
}

// Võtab argumendiks täisarvu ning
// pöörab nii kaua paremale (millisekundites)
void turnRight(int n) {
  servoLeft.write(180);
  servoRight.write(180);
  delay(n);
}

// Võtab argumendiks täisarvu ning
// pöörab nii kaua vasakule (millisekundites)
void turnLeft(int n) {
  servoLeft.write(0);
  servoRight.write(0);
  delay(n);
}

// Peatab roboti
void stopRobot() {
  servoLeft.write(93);
  servoRight.write(93);
}
